/* =============================================================================
 *
 *  Description: This is a C implementation for Thread Hibernate
 *
 * -----------------------------------------------------------------------------
 *  Comments:
 *
 * ===========================================================================*/

/* Get access to any of the VDK features & datatypes used */
#include <sys/exception.h>
#include <sysreg.h>
#include <cdefBF533.h>
#include "Hibernate.h"

#define pFlashA_PortB_Data	(volatile unsigned char *)0x20270005
#define	pSDRAM_PTR			(volatile unsigned long *)0x000000
#define	CLEAR_RTC_IRQS		0x807F		// Write-1-To-Clear all RTC interrupts

void Close_Peripherals(void);
void Save_System(void);
void Hibernate(void);
void Open_Peripherals(void);

void
Hibernate_RunFunction(void **inPtr)
{
    /* Put the thread's "main" Initialization HERE */

    while (1)
    {
        /* Put the thread's "main" body HERE */
        VDK_PendSemaphore(kPushButton, 0);
        Close_Peripherals();
    	Save_System();
		Hibernate();
		Open_Peripherals();
		VDK_PostSemaphore(kBlink);
        /* Use a "break" instruction to exit the "while (1)" loop */
    }

    /* Put the thread's exit from "main" HERE */
    /* A thread is automatically Destroyed when it exits its run function */
}

int
Hibernate_ErrorFunction(void **inPtr)
{

    /* TODO - Put this thread's error handling code HERE */

      /* The default ErrorHandler goes to KernelPanic */

	VDK_CThread_Error(VDK_GetThreadID());
	return 0;
}

void
Hibernate_InitFunction(void **inPtr, VDK_ThreadCreationBlock *pTCB)
{
    /* Put code to be executed when this thread has just been created HERE */

    /* This routine does NOT run in new thread's context.  Any non-static thread
     *   initialization should be performed at the beginning of "Run()."
     */
}

void
Hibernate_DestroyFunction(void **inPtr)
{
    /* Put code to be executed when this thread is destroyed HERE */

    /* This routine does NOT run in the thread's context.  Any VDK API calls
     *   should be performed at the end of "Run()."
     */
}

/* ========================================================================== */

void Hibernate(void)
{
    long key = 0x12345678;
    int temp_irq_val;

    //Save key to location 0x3FEC
    *(pSDRAM_PTR+0xFFB) = key;
    //Save SP to location 0x3FF0
    *(pSDRAM_PTR+0xFFC) = sysreg_read(reg_SP);
    //Save FP to location 0x3FF4
    *(pSDRAM_PTR+0xFFD) = sysreg_read(reg_FP);
	//Save Hibernate Return Addres to location 0x3FFC
    *(pSDRAM_PTR+0xFFF) = sysreg_read(reg_RETS);
        
	*pEBIU_SDGCTL |= SRFS;				// Enable self-refresh on SDRAM
	while(!(*pEBIU_SDSTAT & SDSRA));	// Wait For self refresh active
	
	temp_irq_val = cli();
	Configure_RTC();
	*pRTC_ISTAT = CLEAR_RTC_IRQS;	// Clear any pending IRQs
    *pSIC_IWR |= 0x80;				// Enable RTC wakeup
    idle();							// We should break out of this idle at the next RTC wakeup
    *pRTC_SWCNT = 0x0200;			// Reset Stopwatch for (80/32768) Seconds
    while (!(*pRTC_ISTAT & 0x8000));// Wait until all writes have completed
    *pRTC_ISTAT = CLEAR_RTC_IRQS;	// Acknowledge RTC Interrupt Request
    
	*pFIO_FLAG_C = 0x80;			// set PF7 low for profiling purposes

	*pVR_CTL = (*pVR_CTL & 0xFFFC) | 0x8100;	// VROUT = b#00 For Hibernate, Enable SCKELOW

// This sequence MUST be performed in order for PLL_CTL and/or VR_CTL changes to take effect
// The Blackfin will go into Hibernate once the VR has been programmed. 
// Both the RTC wake-up and the #RESET wake-up will reboot the part.
	//temp_irq_val = cli();
	idle();
	idle();
	//sti(temp_irq_val);

	*pFlashA_PortB_Data |= 0x10;	// We should never reach this point.  Turn on LED to indicate that something went wrong.
	while(1);
	
}
